/*####################################################################
	This class conducts a nonlinear least squares adjustment given
	sets of measured and control point data using the iterative
	nonlinear direct solution. An inverse matrix is calculated using
	standard Gaussian elimination.
####################################################################*/



#ifndef ADJUSTNONLINEAR_H
#define ADJUSTNONLINEAR_H

#include "datastructs.h"
#include "mainwindow.h"
#include "matrix.h"
#include "math.h"


class AdjustNonLinear
{
//############################################
//#####  AdjustNonLinear Types  ##############
//############################################
public:

//############################################
//#####  AdjustNonLinear Functions  ##########
//############################################

public:
	double adjust( int npts, p3d* mpts, p3d* cpts, p3d* &tpts, params* pa, double &rms );			//main host function

private:
	int	initialGuess( params* &pa, p3d* &tpts, p3d* mpts, int n, double* &t );						//computes the initial guess for the parameters
	double iterate( int npts, p3d* mpts, p3d* cpts, p3d* &tpts, params* &pa, double* &tr );
	void transform( int n, p3d* mpts, p3d* &tpts, params* pa, double* tr );
	double angleRangeCorrect(double a);																//force the angle to be within 2pi
	double** matrixMult( double** m1, int nr1, int nc1, double** m2, int nr2, int nc2 );			//executes matrix multiplication
	double** matrixMultTrans( double** m1, int nr1, int nc1, double** m2, int nr2, int nc2 );		//same as matrixMult but with the first matrix transposed

//############################################
//#####  AdjustNonLinear Variables  ##########
//############################################
public:
	double** RotMat;

};

#endif // ADJUSTNONLINEAR_H
