#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QtGui/QMainWindow>
#include "datastructs.h"
#include "adjustnonlinear.h"

class QFile;
class QLabel;
class QPushButton;
class QVBoxLayout;
class QPlainTextEdit;
//class adj_nonlin;

class MainWindow : public QMainWindow
{
    Q_OBJECT

//############################################
//#####  MainWindow Functions  ###############
//############################################
public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();
private:
	/*	Imports a set of point coordinates from a simple CSV file.
		Coordinate order of "X,Y,Z" is required. Text after the Z
		coordinates	will be ignored. Comment lines must begin with
		a "#" character. */

	int importPointCSV(p3d* &p, bool doNptsCheck, QString title);
	void radToDMS( double angleRad, double &d, double &m, double &s );

//############################################
//#####  MainWindow Variables  ###############
//############################################
private:
	QWidget* centralWidget;
	QVBoxLayout* layout_Main;
	QPlainTextEdit* text_ViewLog;
	QPushButton* pushbutton_Start;
	QFile* readf;

	int npts;							// number of points imported (note: number of control points and measured points must match)
	p3d* ctrlpts;						// control point coordinates
	p3d* measpts;						// field-measured points coordinates
	p3d* transpts;						// measured point coordinates after transformation
	params* trans_params;				// transformation parameters

//############################################
//#####  MainWindow Signals & Slots  #########
//############################################
signals:
	void userUpdated();

private slots:
	/*	Activates when the Start button is clicked. Begins the adjustment. */
	void pushbutton_Start_onClicked();
	/*	Convenience function to update the user. Send a QString to be displayed
		in a console window to the user. Only displays plain text, though HTML
		formatting is accepted.
		Emits the userUpdated() signal to indicate a change in the label. */
	void userUpdate(QString t);
	void userUpdate(QString t, bool clear);
	void userUpdateFormatted(QString t);
	void userUpdateFormatted(QString t, bool clear);
};

#endif // MAINWINDOW_H
