#include "matrix.h" 


//
// Matrix Addition
//
double**
addMatrices( double** m1, double** m2, int ncols, int nrows )
{
	double** mresult = new double*[ncols];	//create an array of "n" pointers that act as columns
	for (int i = 0; i < ncols; i++)			//create "n" arrays of "m" doubles
	{
		mresult[i] = new double[nrows];		//i.e., in each column, you should have "m" rows. syntax: mresult[col][row]
	}
	
	//add matrix values row by row
	for ( int j = 0; j < nrows; j++ ) //cycle through rows j
	{
		for ( int i = 0; i < ncols; i++ ) //cycle through columns i
		{
			mresult[i][j] = m1[i][j] + m2[i][j];
		} //for each column within a row
	} //for each row

	return mresult;
}

//
// Matrix Subtraction
//
double**
subMatrices( double** m1, double** m2, int ncols, int nrows )
{
	double** mresult = new double*[ncols];	//create an array of "n" pointers that act as columns
	for (int i = 0; i < ncols; i++)			//create "n" arrays of "m" doubles
	{
		mresult[i] = new double[nrows];		//i.e., in each column, you should have "m" rows. syntax: mresult[col][row]
	}
	
	//subtract matrix values row by row
	for ( int j = 0; j < nrows; j++ ) //cycle through rows j
	{
		for ( int i = 0; i < ncols; i++ ) //cycle through columns i
		{
			mresult[i][j] = m1[i][j] - m2[i][j];
		} //for each column within a row
	} //for each row

	return mresult;
}

//
// Matrix Multiplication of a nxm matrix
//
double**
multiMatrices( double** m1, double** m2, int ncol1, int nrow1, int ncol2, int nrow2 )
{
	double** mresult = new double*[ncol2];	//create an array of "n" pointers that act as columns
	for (int i = 0; i < ncol2; i++)			//create "n" arrays of "m" doubles
	{
		mresult[i] = new double[nrow1];		//i.e., in each column, you should have "m" rows. syntax: mresult[col][row]
	}
	
	//work through the result matrix
	for ( int j = 0; j < nrow1; j++ ) //cycle through rows j of the result matrix
	{
		for ( int i = 0; i < ncol2; i++ ) //cycle through columns i of the result matrix
		{
			mresult[i][j] = 0; //ensure that the value is zero before starting
			//calculate the value at i,j in the result matrix by cycling through the input matrices
			for ( int k = 0; k < ncol1; k++ )
			{
				mresult[i][j] += m1[k][j] * m2[i][k];
			} //for product summation (input)
		} //for each column within a row (output)
	} //for each row (output)

	return mresult;
}

//
// Matrix Transposition
//
double**
transposeMatrix( double** m1, int ncols, int nrows )
{
	double** mresult = new double*[ncols];	//create an array of "n" pointers that act as columns
	for (int i = 0; i < ncols; i++)			//create "n" arrays of "m" doubles
	{
		mresult[i] = new double[nrows];		//i.e., in each column, you should have "m" rows. syntax: mresult[col][row]
	}
	
	//work through the result matrix row by row by column
	for ( int j = 0; j < nrows; j++ ) //cycle through rows j of the result matrix
	{
		for ( int i = 0; i < ncols; i++ ) //cycle through columns i of the result matrix
		{
			//flip the input matrix arcoss its main diagonal
			mresult[i][j] = m1[j][i]; 
		} //for each column within a row (output)
	} //for each row (output)

	return mresult;
}
