#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QtGui/QMainWindow>
#include "scancontroller.h"
#include "vzconfig.h"
#include "myplaintextedit.h"

class ScanController;
class VZConfig;
class MyPlainTextEdit;
class QVBoxLayout;
class QHBoxLayout;
class QFormLayout;
class QPushButton;
class QAction;
class QComboBox;
class QLabel;
class QButtonGroup;
class QGroupBox;
class QRadioButton;
class QDoubleSpinBox;
class QSpinBox;
class QAbstractButton;
class QCheckBox;
class QDockWidget;
class QGridLayout;
class QProgressBar;

class MainWindow : public QMainWindow
{
    Q_OBJECT

	//############################################
	//#####  MainWindow Functions  ###############
	//############################################
public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();
	void finalizeImport();
	void infoUpdate(QString info);

private:
	void refreshFormDisplay();
	void setupGUI();
	void writeScan();

	//############################################
	//#####  MainWindow Signals/Slots  ###########
	//############################################
signals:
	void busy(bool);

public slots:
	void setRunMode(int m);

protected slots:
	void changeImageMode();
	void on_IsBusy(bool busy);
	void on_floatingDock(bool isFloating);

	void on_ScanBegin();
	void on_SessionFinished(bool success);
	void on_action_RetrieveProjectList_triggered();
	void on_pushb_CalcScanTime_clicked();
	void on_pushb_OpenLog_clicked();
	void on_pushb_Start_clicked();
	void on_combo_ProjectList_currentIndexChanged(QString t);
	void on_combo_ProjectList_editTextChanged(QString t);
	void on_combo_PresetSelect_currentIndexChanged(int i);
	void on_bgroup_Pattern_buttonClicked(QAbstractButton* b);

	void timer_ScanStart();
	void timer_ScanContinue();
	void timer_ScanCancelled();
	void timer_ScanFinished(bool success);
	void timer_CamStart(int numPhotos);
	void timer_CamContinue();
	void timer_CamCancelled();
	void timer_CamFinished(bool success);

	//############################################
	//#####  MainWindow Widgets  #################
	//############################################
public:
	QAction* action_ScanBegin;
	QAction* action_RetrieveProjectList;

protected:
	QWidget* centralWidget;					//the core component of the MainWindow that holds every GUI object
	QVBoxLayout* layout_Main;				//the core framework of the centralWidget
		QFormLayout* layout_Config;
			QHBoxLayout* layout_Config_Projects;
				QComboBox* combo_Config_ProjectList;
				QPushButton* pushb_Config_RetrieveList;
				QCheckBox* check_Debug;
			QComboBox* combo_Config_Storage;
			QComboBox* combo_Config_MeasProg;
			QHBoxLayout* layout_Config_Atmos;
				QLabel* label_Config_Temp;
				QSpinBox* spinbox_Config_Temp;
				QLabel* label_Config_Pres;
				QSpinBox* spinbox_Config_Pres;
				QLabel* label_Config_RH;
				QSpinBox* spinbox_Config_RH;
		QFormLayout* layout_Pattern;
			QGroupBox* groupbox_Pattern;
			QButtonGroup* bgroup_Pattern_ManualPreset;
				QRadioButton* radio_PatternPreset;
				QComboBox* combo_Pattern_PresetSelect;
				QRadioButton* radio_PatternUser;
			QGridLayout* layout_Pattern_Details;
				QLabel* label_Pattern_Vert;
				QLabel* label_Pattern_VStart;
				QLabel* label_Pattern_VStop;
				QLabel* label_Pattern_VIncr;
				QLabel* label_Pattern_Horiz;
				QLabel* label_Pattern_HStart;
				QLabel* label_Pattern_HStop;
				QLabel* label_Pattern_HIncr;
				QDoubleSpinBox* spinbox_Pattern_VStart;
				QDoubleSpinBox* spinbox_Pattern_VStop;
				QDoubleSpinBox* spinbox_Pattern_VIncr;
				QDoubleSpinBox* spinbox_Pattern_HStart;
				QDoubleSpinBox* spinbox_Pattern_HStop;
				QDoubleSpinBox* spinbox_Pattern_HIncr;
		QHBoxLayout* layout_Options;
			QCheckBox* check_Images;
			QCheckBox* check_HDR;
			QCheckBox* check_RealTime;
	QHBoxLayout* layout_Buttons;
		QPushButton* pushb_Start;
		QPushButton* pushb_CalcScanTime;
		QPushButton* pushb_OpenLog;

	/* docked widgets */
	QDockWidget* southDock;
		MyPlainTextEdit* text_InfoLog;

	/* status bar widgets */
	QTimer* scanTimer;
	QProgressBar* pbar_ScanProgress;
	QPushButton* pushb_ScanStop;


	//############################################
	//#####  MainWindow Variables  ###############
	//############################################
public:
	ScanController* vzcontrol;
	VZConfig*	newConfig;

protected:
	QStringList combo_Config_ProjectList_options;
	/* Program operation mode, set by command-line arguments
	   0 = Basic mode (default) - command-line arguments are ignored, user input required
	   1 = Auto-start - scanner controlled entirely by command-line arguments, user input disabled
	   2 = Wait - command-line arguments are accepted, user input allowed
	*/
	int runMode;

	// make ScanController a friend of MainWindow so that it can interact with the UI
	friend class ScanController;

};

#endif // MAINWINDOW_H
